//-------------------------------------------------------------------------
//-------------------------------------------------------------------------
//-------Festlegungen------------------------------------------------------
//-------------------------------------------------------------------------
//-------------------------------------------------------------------------

#define  F_CPU 3686400     // Taktfrequenz des myAVR-Boards

//-------------------------------------------------------------------------
//-------------------------------------------------------------------------
//-------Einfgen der Bibliotheken-----------------------------------------
//-------------------------------------------------------------------------
//-------------------------------------------------------------------------

#include <avr/io.h>        // Grundbibliothek
#include <stdio.h>         // Bibliothek fr Sprnge
#include "rc5.h" //prototypen

//-------------------------------------------------------------------------
//-------------------------------------------------------------------------
//-------Variablendeklaration----------------------------------------------
//-------------------------------------------------------------------------
//-------------------------------------------------------------------------


//-------------------------------------------------------------------------
//-------------------------------------------------------------------------
//-------Funktionen--------------------------------------------------------
//-------------------------------------------------------------------------
//-------------------------------------------------------------------------

/**************************************************************************
* Initialisierung des Empfangsports:                                      *
*Diese Funktion initialisiert den Port B.0 als Eingang fr den Empfang des*
*RC-5 Protokolls.                                                         *
**************************************************************************/
void rc5_receive_init()
{
  //Eingnge
  DDRB &= ~(1<<0);      //B.0 = Eingang
  PORTB|=  (1<<0);      //B.0 = PullUP
}


/**************************************************************************
*Empfangen eines Rc-5 Telegramms:                                         *
*Fr die Verwendung dieser Funktion im Hauptprogramm wird                 *
*die Verwendung folgender Syntax empfohlen:                               *
*                                                                         *
*if (bit_is_clear(PINB,0)) // Erkennung Telegrammbeginn                   *
*{                                                                        *
*  rc5_receive();                                                         *
*}                                                                        *
**************************************************************************/
int rc5_receive()
{
  /*In diese Variable wird das empfangene RC5 Telegramm vorlufig hineinge-
  schrieben, um anschlieend zu testen, ob das Telegramm fehler-
  frei empfangen wurde.*/
  int empfangsfach;
  
  /*In diese Variable wird eine Kopie des PINB Registers geschrieben,um 
  eine Flanke an dem Eingang des Empfngermuduls feststellen zu knnen.
  (sowohl eine negativ als auch eine positive Flanke)*/
  int kopie_pinb;

  /*Diese Variable wird ausschlielich dafr verwendet, Zuweisungen auszu-
  fhren um Wartezeiten zu realisieren.*/
  int dummy;

  /*Dies ist eine Zhlvariable, die fr die Realisierung einer Wartezeit
  und einer Pulsweitenmodulation verwendet wird.*/
  int wartezeit;

  empfangsfach=1;    // Eintrag des 1. Startbits in
  anfang:
  for (wartezeit = 0;wartezeit<=213;wartezeit++)
                               // ca. 3/4 Periodendauer
  {
    dummy=1;
  }     
  kopie_pinb=PINB & 1;       // Kopie fr anschl. Fl.-Erkennung
  for (wartezeit= 0;wartezeit<=96;wartezeit++)
                                             // ca. 1/2 Periodendauer
  {
    if ((kopie_pinb^(PINB & 1))==1)
    {
      if (kopie_pinb==0)
      {
        empfangsfach=empfangsfach*2;
      }

      if (kopie_pinb==1)
      {
        empfangsfach=empfangsfach*2;
        empfangsfach|=1;
      }

      goto anfang;
    }
  } 
    

  if((empfangsfach&0x2000)==0x2000)
  {
    return (empfangsfach);
  }
  else
  {
    return 0;
  }
}
/**************************************************************************
* Initialisierung des Empfangsports:                                      *
*Diese Funktion initialisiert den Port B.0 als Eingang fr den Empfang des*
*RC-5 Protokolls.                                                         *
**************************************************************************/
void rc5_send_init()
{
  DDRB   |=  (1<<1);     //B.1= Ausgang fr Sendediode
}
/**************************************************************************
*Senden eines Rc5 Telegramms                                              *
*Diese Funktion gibt das zu sendende Telegramm an PortB.1 aus             *
*                                                                         *
*ACHTUNG!!!! FUNKTIONIERT NUR MIT DEM ATMEGA8!!!                          *
*Da alle Zeiten in der Senderoutine auf des Taktfrequenz des ATmega8 ba-  *
*sieren und keine fertigen Wartezeiten verwendet werden, sind Trgerfre-  *
*quenz und Periodendauer auch nur mit dem ATmega8 (evtl. anderer Prozessor*
*mit gleicher Taktzeit) entsprechend dem RC5-Protokoll.                   *
*                                                                         *
*Fr die Verwendung dieser Funktion im Hauptprogramm wird                 *
*die Verwendung folgender Syntax empfohlen:                               *
*                                                                         *
*if(sendefach>0) // Abfrage auf Inhalt in dem sendefach                   *
*{                                                                        *
*  rc5_dend(sendefach);  ........................................         *
*                                                                         *
*  //Da das RC5-Telegramm 14 Bit hat, das Sendefach jedoch 16, muss dieses*
*  //nach dem Sendevorgang gelscht werden, damit nicht ein erneuter      *
*  //sendevorgang ausgelst wird.                                         *
*  sendefach=0;...........................................................*
*}                                                                        *
**************************************************************************/
void rc5_send(int data)
{

  int gesendete_bits;
  int pwm_takt=0; 

  while (gesendete_bits<14)
  {
    for(pwm_takt=0;pwm_takt<=139;pwm_takt++)
    {
      if (pwm_takt<=69)
      {
		    if ((data&0x2000)==0x2000)
		    {
          PORTB &=~(1<<1);    
		    }
		    else
		    {
          PORTB ^= (1<<1);
		    }
      }
      else
      {
        if ((data&0x2000)==0x2000)
		    {
          PORTB ^= (1<<1);    
		    }
		      else
		    {
          PORTB &=~(1<<1);
		    }
      }
    }
		gesendete_bits++;
    data=(data<<1);
  }
  data=0;
	gesendete_bits=0;
}




